@@ -1,18 +1,19 @@
 package Memcached::Client::Connection;
 BEGIN {
-  $Memcached::Client::Connection::VERSION = '1.04';
+  $Memcached::Client::Connection::VERSION = '1.05';
 }
 # ABSTRACT: Class to manage Memcached::Client server connections
 
 use strict;
 use warnings;
+use AnyEvent qw{};
 use AnyEvent::Handle qw{};
 use Memcached::Client::Log qw{DEBUG INFO};
 
 
 sub new {
     my ($class, $server, $preparation) = @_;
-    my $self = bless {prepare => $preparation, queue => [], server => $server}, $class;
+    my $self = bless {attempts => 0, last => 0, prepare => $preparation, requests => 0, queue => [], server => $server}, $class;
     return $self;
 }
 
@@ -31,16 +32,31 @@ sub connect {
                                                on_connect => sub {
                                                    my ($handle, $host, $port) = @_;
                                                    DEBUG "C [%s]: connected", $self->{server};
+                                                   $self->{attempts} = 0;
+                                                   $self->{last} = 0;
+                                                   $self->{requests} = 0;
                                                    $callback->() if ($callback);
                                                    $self->dequeue;
                                                },
                                                on_error => sub {
                                                    my ($handle, $fatal, $message) = @_;
-                                                   INFO "C [%s]: %s error %s", $self->{server}, ($fatal ? "fatal" : "non-fatal"), $message;
+                                                   my $last = $self->{last} ? AE::time - $self->{last} : 0;
+                                                   my $pending = scalar @{$self->{queue}};
                                                    # Need this here in case connection fails
-                                                   $callback->() if ($callback);
-                                                   delete $self->{handle};
-                                                   $self->fail;
+                                                   if ($message eq "Broken pipe") {
+                                                       DEBUG "Requeueing broken pipe for %s", $self->{server};
+                                                       delete $self->{handle};
+                                                       $self->connect;
+                                                   } elsif ($message eq "Connection timed out" and ++$self->{attempts} < 5) {
+                                                       DEBUG "Requeueing connection timeout for %s", $self->{server};
+                                                       delete $self->{handle};
+                                                       $self->connect ();
+                                                   } else {
+                                                       INFO "C [%s]: %s, %d attempts, %d completed, %d pending, %f last", $self->{server}, $message, $self->{attempts}, $self->{requests}, $pending, $last;
+                                                       $callback->() if ($callback);
+                                                       delete $self->{handle};
+                                                       $self->fail;
+                                                   }
                                                },
                                                on_prepare => sub {
                                                    my ($handle) = @_;
@@ -56,6 +72,7 @@ sub enqueue {
     my ($self, $request, $failback) = @_;
     $self->connect unless ($self->{handle});
     DEBUG "C [%s]: queuing request", $self->{server};
+    $self->{last} = AE::time;
     push @{$self->{queue}}, {request => $request, failback => $failback};
     $self->dequeue;
     return 1;
@@ -70,6 +87,7 @@ sub dequeue {
     DEBUG "C [%s]: executing", $self->{server};
     $self->{executing}->{request}->($self->{handle},
                                     sub {
+                                        $self->{requests}++;
                                         DEBUG "C [%s]: done with request", $self->{server};
                                         delete $self->{executing};
                                         $self->dequeue;
@@ -108,7 +126,7 @@ Memcached::Client::Connection - Class to manage Memcached::Client server connect
 
 =head1 VERSION
 
-version 1.04
+version 1.05
 
 =head1 SYNOPSIS
 
